@@ -35,10 +35,9 @@
 #include "common/nmv-ustring.h"
 #include "common/nmv-dynamic-module.h"
 #include "common/nmv-safe-ptr-utils.h"
+#include "common/nmv-address.h"
+#include "common/nmv-asm-instr.h"
 #include "nmv-i-conf-mgr.h"
-#include "nmv-i-lang-trait.h"
-
-NEMIVER_BEGIN_NAMESPACE (nemiver)
 
 using nemiver::common::SafePtr;
 using nemiver::common::DynamicModule;
@@ -48,11 +47,19 @@ using nemiver::common::ObjectRef;
 using nemiver::common::ObjectUnref;
 using nemiver::common::UString;
 using nemiver::common::Object;
+using nemiver::common::Address;
+using nemiver::common::AsmInstr;
+using nemiver::common::MixedAsmInstr;
+using nemiver::common::Asm;
+using nemiver::common::DisassembleInfo;
 using std::vector;
 using std::string;
 using std::map;
 using std::list;
 
+NEMIVER_BEGIN_NAMESPACE (nemiver)
+
+class ILangTrait;
 class IDebugger;
 typedef SafePtr<IDebugger, ObjectRef, ObjectUnref> IDebuggerSafePtr;
 
@@ -77,20 +84,34 @@ public:
     typedef unsigned int register_id_t;
 
     /// \brief a breakpoint descriptor
-    class BreakPoint {
-        int m_number;
-        bool m_enabled;
-        UString m_address;
-        UString m_function;
-        UString m_file_name;
-        UString m_file_full_name;
-        int m_line;
-        UString m_condition;
-        int m_nb_times_hit;
-
+    class Breakpoint {
     public:
 
-        BreakPoint () {clear ();}
+        enum Type {
+            UNDEFINED_TYPE = 0,
+            STANDARD_BREAKPOINT_TYPE,
+            WATCHPOINT_TYPE,
+            COUNTPOINT_TYPE
+        };
+
+    private:
+        int m_number;
+        bool m_enabled;
+        Address m_address;
+        string m_function;
+        string m_expression;
+        UString m_file_name;
+        UString m_file_full_name;
+        string m_condition;
+        Type m_type;
+        int m_line;
+        int m_nb_times_hit;
+        int m_ignore_count;
+        bool m_is_read_watchpoint;
+        bool m_is_write_watchpoint;
+
+    public:
+        Breakpoint () {clear ();}
 
         /// \name accessors
 
@@ -101,11 +122,15 @@ public:
         bool enabled () const {return m_enabled;}
         void enabled (bool a_in) {m_enabled = a_in;}
 
-        const UString& address () const {return m_address;}
-        void address (const UString &a_in) {m_address = a_in;}
+        const Address& address () const {return m_address;}
+        Address& address () {return m_address;}
+        void address (const string &a_in) {m_address = a_in;}
 
-        const UString& function () const {return m_function;}
-        void function (const UString &a_in) {m_function = a_in;}
+        const string& function () const {return m_function;}
+        void function (const string &a_in) {m_function = a_in;}
+
+        const string& expression () const {return m_expression;}
+        void expression (const string &a_expr) {m_expression = a_expr;}
 
         const UString& file_name () const {return m_file_name;}
         void file_name (const UString &a_in) {m_file_name = a_in;}
@@ -116,14 +141,23 @@ public:
         int line () const {return m_line;}
         void line (int a_in) {m_line = a_in;}
 
-        const UString& condition () const {return m_condition;}
-        void condition (const UString &a_cond) {m_condition = a_cond;}
+        const string& condition () const {return m_condition;}
+        void condition (const string &a_cond) {m_condition = a_cond;}
 
         bool has_condition () const {return !m_condition.empty ();}
 
         int nb_times_hit () const {return m_nb_times_hit;}
         void nb_times_hit (int a_nb) {m_nb_times_hit = a_nb;}
 
+        int ignore_count () const {return m_ignore_count;}
+        void ignore_count (int a) {m_ignore_count = a;}
+
+        bool is_read_watchpoint () const {return m_is_read_watchpoint;}
+        void is_read_watchpoint (bool f) {m_is_read_watchpoint = f;}
+
+        bool is_write_watchpoint () const {return m_is_write_watchpoint;}
+        void is_write_watchpoint (bool f) {m_is_write_watchpoint = f;}
+
         bool is_pending ()
         {
             if (m_address == "<PENDING>") {
@@ -131,11 +165,16 @@ public:
             }
             return false;
         }
+
+        Type type () const {return m_type;}
+        void type (Type a_type) {m_type = a_type;}
+
         /// @}
 
         /// \brief clear this instance of breakpoint
         void clear ()
         {
+            m_type = STANDARD_BREAKPOINT_TYPE;
             m_number = 0;
             m_enabled = false;
             m_address.clear ();
@@ -145,8 +184,11 @@ public:
             m_line = 0;
             m_condition.clear ();
             m_nb_times_hit = 0;
+            m_ignore_count = 0;
+            m_is_read_watchpoint = false;
+            m_is_write_watchpoint = false;
         }
-    };//end class BreakPoint
+    };//end class Breakpoint
 
     /// \brief an entry of a choice list
     /// to choose between a list of overloaded
@@ -216,9 +258,9 @@ public:
 
     /// \brief a function frame as seen by the debugger.
     class Frame {
-        UString m_address;
-        UString m_function_name;
-        map<UString, UString> m_args;
+        Address m_address;
+        string m_function_name;
+        map<string, string> m_args;
         int m_level;
         //present if the target has debugging info
         UString m_file_name;
@@ -226,22 +268,30 @@ public:
         UString m_file_full_name;
         int m_line;
         //present if the target doesn't have debugging info
-        UString m_library;
+        string m_library;
     public:
 
-        Frame () {clear ();}
+        Frame () :
+            m_level (0),
+            m_line (0)
+        {}
 
         /// \name accessors
 
         /// @{
-        const UString& address () const {return m_address;}
-        void address (const UString &a_in) {m_address = a_in;}
+        const Address& address () const {return m_address;}
+        Address& address () {return m_address;}
+        void address (const Address &a_in) {m_address = a_in;}
+        bool has_empty_address () const
+        {
+            return m_address.to_string ().empty ();
+        }
 
-        const UString& function_name () const {return m_function_name;}
-        void function_name (const UString &a_in) {m_function_name = a_in;}
+        const string& function_name () const {return m_function_name;}
+        void function_name (const string &a_in) {m_function_name = a_in;}
 
-        const map<UString, UString>& args () const {return m_args;}
-        map<UString, UString>& args () {return m_args;}
+        const map<string, string>& args () const {return m_args;}
+        map<string, string>& args () {return m_args;}
 
         int level () const {return m_level;}
         void level (int a_level) {m_level = a_level;}
@@ -255,8 +305,8 @@ public:
         int line () const {return m_line;}
         void line (int a_in) {m_line = a_in;}
 
-        const UString& library () const {return m_library;}
-        void library (const UString &a_library) {m_library = a_library;}
+        const string& library () const {return m_library;}
+        void library (const string &a_library) {m_library = a_library;}
 
         /// @}
 
@@ -277,8 +327,20 @@ public:
 
     class Variable;
     typedef SafePtr<Variable, ObjectRef, ObjectUnref> VariableSafePtr;
+    typedef list<VariableSafePtr> VariableList;
     class Variable : public Object {
-        list<VariableSafePtr> m_members;
+        // non copyable.
+        Variable (const Variable &);
+        Variable& operator= (const Variable &);
+
+        bool operator == (Variable &a_other) const;
+        bool operator == (Variable &a_other);
+
+        VariableList m_members;
+        // If this variable was created with a backend counterpart
+        // (e.g: backend side variable objects in GDB), then this
+        // is the name of the backend side counterpart of this variable.
+        UString m_internal_name;
         UString m_name;
         UString m_name_caption;
         UString m_value;
@@ -288,8 +350,27 @@ public:
         //it can be dereferenced. The variable
         //it points to is stored in m_dereferenced
         VariableSafePtr m_dereferenced;
+        unsigned int m_num_expected_children;
+        // The expression with which this variable
+        // Can be referenced in the debugger.
+        // If empty, it can be set by calling
+        // IDebugger::query_variable_path_expr()
+        UString m_path_expression;
 
     public:
+        Variable (const UString &a_internal_name,
+                  const UString &a_name,
+                  const UString &a_value,
+                  const UString &a_type) :
+            m_internal_name (a_internal_name),
+            m_name (a_name),
+            m_value (a_value),
+            m_type (a_type),
+            m_parent (0),
+            m_num_expected_children (0)
+
+        {
+        }
 
         Variable (const UString &a_name,
                   const UString &a_value,
@@ -297,21 +378,92 @@ public:
             m_name (a_name),
             m_value (a_value),
             m_type (a_type),
-            m_parent (0)
+            m_parent (0),
+            m_num_expected_children (0)
 
         {
         }
 
         Variable (const UString &a_name) :
             m_name (a_name),
-            m_parent (0)
+            m_parent (0),
+            m_num_expected_children (0)
         {}
 
         Variable () :
-            m_parent (0)
+            m_parent (0),
+            m_num_expected_children (0)
         {}
 
-        const list<VariableSafePtr>& members () const {return m_members;}
+        const VariableList& members () const {return m_members;}
+
+        /// Returns the Nth member variable, if any.
+        /// Please note that this function has O(n) complexity. So use it with
+        /// care.
+        /// \param a_index, the index of the member variable to return,
+        ///        starting from zero.
+        /// \param a_var the output parameter. Is set if and only if the
+        ///         the function returned true.
+        /// \return true if there was a variable at the specified index,
+        ///  false otherwise.
+        bool get_member_at (int a_index,
+                            VariableSafePtr &a_var) const
+        {
+            int i = 0;
+            for (VariableList::const_iterator it = members ().begin ();
+                 it != m_members.end ();
+                 ++it, ++i) {
+                if (i == a_index) {
+                    a_var = *it;
+                    return true;
+                }
+            }
+            return false;
+        }
+
+        /// \return the sibbling index of the variable node.
+        /// If first sibbling at a given level has index 0.
+        int sibling_index () const
+        {
+            if (!parent ())
+                return 0;
+            VariableList::const_iterator it;
+            int i = 0;
+            for (it = parent ()->members ().begin ();
+                 it != parent ()->members ().end ();
+                 ++it, ++i) {
+                if (*it == this)
+                    return i;
+            }
+            THROW ("fatal: should not be reached");
+        }
+
+        /// Tests value equality between two variables.
+        /// Two variables are considered equal by value if their
+        /// respective memebers have the same values and same type.
+        /// \param a_other the other variable to test against
+        /// \return true if a_other equals the current instance
+        ///         by value.
+        bool equals_by_value (const Variable &a_other) const
+        {
+            if (name () != a_other.name ()
+                || type () != a_other.type ())
+                return false;
+            if (members ().empty () != a_other.members ().empty ())
+                return false;
+            VariableList::const_iterator it0, it1;
+            for (it0 = members ().begin (), it1 = a_other.members ().begin ();
+                 it0 != members ().end ();
+                 ++it0, ++it1) {
+                if (it1 == a_other.members ().end ())
+                    return false;
+                if (!(*it0)->equals_by_value (**it1))
+                    return false;
+            }
+            if (it1 != a_other.members ().end ())
+                return false;
+            return true;
+        }
 
         void append (const VariableSafePtr &a_var)
         {
@@ -320,6 +472,19 @@ public:
             a_var->parent (this);
         }
 
+        /// If this variable was created with a backend counterpart
+        /// (e.g. backend side variable objects in GDB), then this
+        /// returns the name of the backend side counterpart of this variable.
+        /// \return the name of the backend side counterpart of this
+        /// variable. It can be an empty string if there is no backend side
+        /// counterpart variable object created for this object.
+        const UString& internal_name () const {return m_internal_name;}
+
+        /// Set the name of the backend side counterpart object.
+        /// (e.g. variable object in GDB).
+        /// \param a_in the new name of backend side counterpart variable object.
+        void internal_name (const UString &a_in) {m_internal_name = a_in;}
+
         const UString& name () const {return m_name;}
         void name (const UString &a_name)
         {
@@ -337,12 +502,40 @@ public:
         void type (const UString &a_type) {m_type = a_type;}
         void type (const string &a_type) {m_type = a_type;}
 
-        Variable* parent () const {return m_parent;}
+        /// Return true if this instance of Variable has a parent variable,
+        /// false otherwise.
+        bool has_parent () const
+        {
+            return m_parent != 0;
+        }
+
+        /// A getter of the parent Variable of the current instance.
+        const VariableSafePtr parent () const
+        {
+            VariableSafePtr parent (m_parent, true/*add a reference*/);
+            return parent;
+        }
+
+        /// Parent variable setter.
+        /// We don't increase the ref count of the
+        /// parent variable to avoid memory leaks due to refcounting cyles.
+        /// The parent already holds a reference on this instance of
+        /// Variable so we really should not hold a reference on it as
+        /// well.
         void parent (Variable *a_parent)
         {
             m_parent = a_parent;
         }
 
+        /// \return the root parent of this variable.
+        const VariableSafePtr root () const
+        {
+            if (!has_parent ()) {
+                return VariableSafePtr (this, true /*increase refcount*/);
+            }
+            return parent ()->root ();
+        }
+
         void to_string (UString &a_str,
                         bool a_show_var_name = false,
                         const UString &a_indent_str="") const
@@ -351,6 +544,10 @@ public:
                 if (name () != "") {
                     a_str += a_indent_str + name ();
                 }
+
+                if (!internal_name ().empty ()) {
+                    a_str += "(" + internal_name () + ")";
+                }
             }
             if (value () != "") {
                 if (a_show_var_name) {
@@ -363,7 +560,7 @@ public:
             }
             UString indent_str = a_indent_str + "  ";
             a_str += "\n" + a_indent_str + "{";
-            list<VariableSafePtr>::const_iterator it;
+            VariableList::const_iterator it;
             for (it = members ().begin (); it != members ().end (); ++it) {
                 if (!(*it)) {continue;}
                 a_str += "\n";
@@ -376,7 +573,7 @@ public:
         void build_qname (UString &a_qname) const
         {
             UString qname;
-            if (parent () == 0) {
+            if (!parent ()) {
                 a_qname = name ();
                 if (!a_qname.raw ().empty () && a_qname.raw ()[0] == '*') {
                     a_qname.erase (0, 1);
@@ -395,6 +592,21 @@ public:
             }
         }
 
+        void build_qualified_internal_name (UString &a_qname) const
+        {
+            UString qname;
+            if (!parent ()) {
+                a_qname = internal_name ();
+            } else if (parent ()) {
+                parent ()->build_qname (qname);
+                qname.chomp ();
+                qname += "." + name ();
+                a_qname = qname;
+            } else {
+                THROW ("should not be reached");
+            }
+        }
+
         void set_dereferenced (VariableSafePtr a_derefed)
         {
             m_dereferenced  = a_derefed;
@@ -422,7 +634,7 @@ public:
                 return false;
             }
 
-            list<VariableSafePtr>::const_iterator it1, it2;
+            VariableList::const_iterator it1, it2;
             //first make sure our members have the same types as their members
             for (it1=members ().begin (), it2=a_other.members ().begin ();
                  it1 != members ().end ();
@@ -442,8 +654,8 @@ public:
             m_name = a_other.m_name;
             m_name_caption = a_other.m_name_caption;
             m_value = a_other.m_value;
-            list<VariableSafePtr>::iterator it1;
-            list<VariableSafePtr>::const_iterator it2;
+            VariableList::iterator it1;
+            VariableList::const_iterator it2;
             for (it1=m_members.begin (), it2=a_other.m_members.begin ();
                  it1 != m_members.end ();
                  it1++, it2++) {
@@ -457,7 +669,7 @@ public:
             m_name = a_other.m_name;
             m_value = a_other.m_value;
             m_type = a_other.m_type;
-            list<VariableSafePtr>::const_iterator it;
+            VariableList::const_iterator it;
             m_members.clear ();
             for (it = a_other.m_members.begin ();
                  it != a_other.m_members.end ();
@@ -468,6 +680,62 @@ public:
                 append (var);
             }
         }
+
+        unsigned int num_expected_children () const
+        {
+            return m_num_expected_children;
+        }
+
+        void num_expected_children (unsigned int a_in)
+        {
+            m_num_expected_children = a_in;
+        }
+
+        bool has_expected_children () const
+        {
+            return m_num_expected_children != 0;
+        }
+
+        /// \return true if the current variable needs to be unfolded
+        /// by a call to IDebugger::unfold_variable()
+        bool needs_unfolding () const
+        {
+            return (has_expected_children () && members ().empty ());
+        }
+
+        /// Return the descendant of the current instance of Variable.
+        /// \param a_internal_path the internal fully qualified path of the
+        ///        descendant variable.
+        const VariableSafePtr get_descendant
+                                (const UString &a_internal_path) const
+        {
+            VariableSafePtr result;
+            if (internal_name () == a_internal_path) {
+                result.reset (this, true /*take refcount*/);
+                return result;
+            }
+            for (VariableList::const_iterator it = m_members.begin ();
+                 it != m_members.end ();
+                 ++it) {
+                if (*it && (*it)->internal_name () == a_internal_path) {
+                    return *it;
+                }
+                result = (*it)->get_descendant (a_internal_path);
+                if (result) {
+                    return result;
+                }
+            }
+            return result;
+        }
+
+        const UString& path_expression () const
+        {
+            return m_path_expression;
+        }
+        void path_expression (const UString &a_expr)
+        {
+            m_path_expression = a_expr;
+        }
     };//end class Variable
 
     enum State {
@@ -513,6 +781,16 @@ public:
         SIGNAL_RECEIVED
     };//end enum StopReason
 
+    /// Return true if a StopReason represents a reason for exiting.
+    static bool is_exited (enum StopReason a_reason)
+    {
+        if (a_reason == EXITED_SIGNALLED
+            || a_reason == EXITED
+            || a_reason == EXITED_NORMALLY)
+            return true;
+        return false;
+    }
+
     virtual ~IDebugger () {}
 
     /// \name events you can connect to.
@@ -541,7 +819,7 @@ public:
     virtual sigc::signal<void>& detached_from_target_signal () const=0;
 
     virtual sigc::signal<void,
-                        const IDebugger::BreakPoint&,
+                        const IDebugger::Breakpoint&,
                         int /*breakpoint command*/,
                         const UString & /*cookie*/>&
                                      breakpoint_deleted_signal () const=0;
@@ -552,7 +830,7 @@ public:
     /// IDebugger does not cache the list of breakpoints. This must
     /// be fixed at some point.
     virtual sigc::signal<void,
-                         const map<int, IDebugger::BreakPoint>&,
+                         const map<int, IDebugger::Breakpoint>&,
                          const UString& /*cookie*/>&
                                          breakpoints_set_signal () const=0;
 
@@ -578,7 +856,7 @@ public:
 
     virtual sigc::signal<void,
                          int/*thread id*/,
-                         const IDebugger::Frame&/*frame in thread*/,
+                         const IDebugger::Frame *const/*frame in thread*/,
                          const UString& /*cookie*/> &
                                              thread_selected_signal () const=0;
 
@@ -601,10 +879,10 @@ public:
                                             current_frame_signal () const = 0;
 
 
-    virtual sigc::signal<void, const list<VariableSafePtr>&, const UString& >&
+    virtual sigc::signal<void, const VariableList&, const UString& >&
                             local_variables_listed_signal () const = 0;
 
-    virtual sigc::signal<void, const list<VariableSafePtr>&, const UString& >&
+    virtual sigc::signal<void, const VariableList&, const UString& >&
                             global_variables_listed_signal () const = 0;
 
     /// Emitted as the result of the IDebugger::print_variable_value() call.
@@ -649,7 +927,7 @@ public:
     virtual sigc::signal<void,
                          int/*pid*/,
                          const UString&/*target path*/>&
-                                            got_target_info_signal () const=0;
+                                            got_target_info_signal () const = 0;
 
     virtual sigc::signal<void>& running_signal () const=0;
 
@@ -661,34 +939,70 @@ public:
     virtual sigc::signal<void, const UString&/*error message*/>&
                                                     error_signal () const = 0;
 
-    virtual sigc::signal<void, IDebugger::State>& state_changed_signal () const=0;
+    virtual sigc::signal<void, IDebugger::State>&
+                                        state_changed_signal () const = 0;
 
-    virtual sigc::signal<void, const std::map<register_id_t, UString>&, const UString& >&
-                                                 register_names_listed_signal () const=0;
+    virtual sigc::signal<void,
+                         const std::map<register_id_t, UString>&,
+                         const UString& >&
+                                 register_names_listed_signal () const = 0;
+
+    virtual sigc::signal<void,
+                         const std::map<register_id_t, UString>&,
+                         const UString& >&
+                                   register_values_listed_signal () const = 0;
 
-    virtual sigc::signal<void, const std::map<register_id_t, UString>&, const UString& >&
-                                                 register_values_listed_signal () const=0;
     virtual sigc::signal<void,
                          const UString&/*register name*/,
                          const UString&/*register value*/,
                          const UString&/*cookie*/>&
-                                                 register_value_changed_signal () const=0;
-    virtual sigc::signal<void, const std::list<register_id_t>&, const UString& >&
-                             changed_registers_listed_signal () const=0;
+                                  register_value_changed_signal () const = 0;
+
+    virtual sigc::signal<void,
+                         const std::list<register_id_t>&,
+                         const UString& >&
+                                   changed_registers_listed_signal () const = 0;
 
     virtual sigc::signal <void,
                           size_t,/*start address*/
                           const std::vector<uint8_t>&,/*values*/
                           const UString&>&/*cookie*/
-                                          read_memory_signal () const = 0;
+                                 read_memory_signal () const = 0;
     virtual sigc::signal <void,
                           size_t,/*start address*/
                           const std::vector<uint8_t>&,/*values*/
                           const UString& >&
-                                          set_memory_signal () const = 0;
+                                 set_memory_signal () const = 0;
+
+    // TODO: export informations about what file is being disassembled,
+    // what function, which line (if possible) etc.
+    // So that the code receiving the signal can adjust accordingly
+    virtual sigc::signal<void,
+                         const DisassembleInfo&,
+                         const std::list<Asm>&,
+                         const UString& /*cookie*/>&
+                             instructions_disassembled_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableSafePtr, const UString&>&
+                                 variable_created_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableSafePtr, const UString&>&
+                                 variable_deleted_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableSafePtr, const UString&>&
+                                 variable_unfolded_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableSafePtr, const UString&>&
+                                 variable_expression_evaluated_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableList&, const UString&>&
+                                changed_variables_signal () const  = 0;
+
+    virtual sigc::signal<void, VariableSafePtr, const UString&>&
+                assigned_variable_signal () const = 0;
     /// @}
 
-    virtual void do_init (IConfMgrSafePtr &a_conf_mgr) = 0;
+    virtual void do_init (IConfMgrSafePtr a_conf_mgr) = 0;
 
     virtual map<UString, UString>& properties () = 0;
 
@@ -703,11 +1017,13 @@ public:
 
     virtual void set_solib_prefix_path (const UString &a_name) = 0;
 
-    virtual void load_program (const UString &a_prog_with_args,
+    virtual bool load_program (const UString &a_prog,
+                               const vector<UString> &a_args,
                                const UString &a_working_dir) = 0;
 
-    virtual void load_program
-                (const vector<UString> &a_argv,
+    virtual bool load_program
+                (const UString &a_prog,
+                 const vector<UString> &a_argv,
                  const UString &working_dir,
                  const vector<UString> &a_source_search_dirs,
                  const UString &a_tty_path="") = 0;
@@ -719,14 +1035,18 @@ public:
                                    const UString &a_tty_path="") = 0;
 
     virtual bool attach_to_remote_target (const UString &a_host,
-                                          int a_port) = 0;
+                                          unsigned a_port) = 0;
 
     virtual bool attach_to_remote_target (const UString &a_serial_line) = 0;
 
     virtual void detach_from_target (const UString &a_cookie="") = 0;
 
+    virtual void disconnect_from_remote_target (const UString &a_cookie = "") = 0;
+
     virtual bool is_attached_to_target () const =0;
 
+    virtual void set_tty_path (const UString &a_tty_path) = 0;
+
     virtual void add_env_variables (const map<UString, UString> &a_vars) = 0;
 
     virtual map<UString, UString>& get_env_variables () = 0;
@@ -735,7 +1055,9 @@ public:
 
     virtual void get_target_info (const UString &a_cookie="") = 0;
 
-    virtual ILangTraitSafePtr get_language_trait () = 0;
+    virtual ILangTrait& get_language_trait () = 0;
+
+    virtual bool is_variable_editable (const VariableSafePtr a_var) const = 0;
 
     virtual void do_continue (const UString &a_cookie="") = 0;
 
@@ -749,30 +1071,35 @@ public:
 
     virtual void exit_engine () = 0;
 
-    virtual void step_in (const UString &a_cookie="") = 0;
-
     virtual void step_over (const UString &a_cookie="") = 0;
 
+    virtual void step_in (const UString &a_cookie="") = 0;
+
     virtual void step_out (const UString &a_cookie="") = 0;
 
-    virtual void step_instruction (const UString &a_cookie="") = 0;
+    virtual void step_over_asm (const UString &a_cookie="") = 0;
+
+    virtual void step_in_asm (const UString &a_cookie="") = 0;
 
     virtual void continue_to_position (const UString &a_path,
                                        gint a_line_num,
                                        const UString &a_cookie="") = 0;
+
     virtual void set_breakpoint (const UString &a_path,
                                  gint a_line_num,
-                                 const UString &a_condition="",
-                                 const UString &a_cookie="") = 0;
+                                 const UString &a_condition= "",
+                                 gint a_ignore_count = 0,
+                                 const UString &a_cookie = "") = 0;
+
     virtual void set_breakpoint (const UString &a_func_name,
-                                 const UString &a_condition="",
-                                 const UString &a_cookie="") = 0;
-    virtual void set_catch (const UString &a_event,
-                            const UString &a_cookie="") = 0;
+                                 const UString &a_condition = "",
+                                 gint a_ignore_count = 0,
+                                 const UString &a_cookie = "") = 0;
 
-    virtual void list_breakpoints (const UString &a_cookie="") = 0;
-
-    virtual const map<int, BreakPoint>& get_cached_breakpoints () = 0;
+    virtual void set_breakpoint (const Address &a_address,
+                                 const UString &a_condition = "",
+                                 gint a_ignore_count = 0,
+                                 const UString &a_cookie = "") = 0;
 
     virtual void enable_breakpoint (gint a_break_num,
                                     const UString &a_cookie="") = 0;
@@ -780,10 +1107,43 @@ public:
     virtual void disable_breakpoint (gint a_break_num,
                                      const UString &a_cookie="") = 0;
 
+    virtual void set_breakpoint_ignore_count
+                                        (gint a_break_num,
+                                         gint a_ignore_count,
+                                         const UString &a_cookie = "") = 0;
+
+    virtual void set_breakpoint_condition (gint a_break_num,
+                                           const UString &a_condition,
+                                           const UString &a_cookie = "") = 0;
+
+    virtual void enable_countpoint (gint a_break_num,
+                                    bool a_flag,
+                                    const UString &a_cookie ="") = 0;
+
+    virtual bool is_countpoint (gint a_break_num) const = 0;
+
+    virtual bool is_countpoint (const Breakpoint &a_breakpoint) const = 0;
+
     virtual void delete_breakpoint (const UString &a_path,
                                     gint a_line_num,
                                     const UString &a_cookie="") = 0;
 
+    virtual void set_watchpoint (const UString &a_expression,
+                                 bool a_write = true,
+                                 bool a_read = false,
+                                 const UString &a_cookie = "") = 0;
+
+    virtual void set_catch (const UString &a_event,
+                            const UString &a_cookie="") = 0;
+
+    virtual void list_breakpoints (const UString &a_cookie="") = 0;
+
+    virtual const map<int, Breakpoint>& get_cached_breakpoints () = 0;
+
+    virtual bool get_breakpoint_from_cache (int a_num,
+                                            IDebugger::Breakpoint &a_bp)
+        const = 0;
+
     virtual void choose_function_overload (int a_overload_number,
                                            const UString &a_cookie="") = 0;
 
@@ -798,6 +1158,8 @@ public:
     virtual void select_thread (unsigned int a_thread_id,
                                 const UString &a_cookie="") = 0;
 
+    virtual unsigned int get_current_thread () const = 0;
+
     virtual void select_frame (int a_frame_id,
                                const UString &a_cookie="") = 0;
 
@@ -854,6 +1216,92 @@ public:
             const std::vector<uint8_t>& a_bytes,
             const UString& a_cookie="") = 0;
 
+    typedef sigc::slot<void,
+                       const DisassembleInfo&,
+                       const std::list<Asm>& > DisassSlot;
+
+    virtual void disassemble (size_t a_start_addr,
+                              bool a_start_addr_relative_to_pc,
+                              size_t a_end_addr,
+                              bool a_end_addr_relative_to_pc,
+                              bool a_pure_asm = true,
+                              const UString &a_cookie = "") = 0;
+
+    virtual void disassemble (size_t a_start_addr,
+                              bool a_start_addr_relative_to_pc,
+                              size_t a_end_addr,
+                              bool a_end_addr_relative_to_pc,
+                              const DisassSlot &a_slot,
+                              bool a_pure_asm = true,
+                              const UString &a_cookie = "") = 0;
+
+    virtual void disassemble_lines (const UString &a_file_name,
+                                    int a_line_num,
+                                    int a_nb_disassembled_lines,
+                                    bool a_pure_asm = true,
+                                    const UString &a_cookie = "") = 0;
+
+    virtual void disassemble_lines (const UString &a_file_name,
+                                    int a_line_num,
+                                    int a_nb_disassembled_lines,
+                                    const DisassSlot &a_slot,
+                                    bool a_pure_asm = true,
+                                    const UString &a_cookie = "") = 0;
+
+    typedef sigc::slot<void, const VariableSafePtr> ConstVariableSlot;
+    typedef sigc::slot<void, const VariableList> ConstVariableListSlot;
+    typedef sigc::slot<void, const UString&> ConstUStringSlot;
+
+    virtual void create_variable (const UString &a_name,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void create_variable (const UString &a_name,
+                                  const ConstVariableSlot &a_slot,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void delete_variable (const VariableSafePtr a_var,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void delete_variable (const VariableSafePtr a_var,
+                                  const ConstVariableSlot&,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void unfold_variable (VariableSafePtr a_var,
+                                  const UString &a_cookie = "") = 0;
+    virtual void unfold_variable
+                (VariableSafePtr a_var,
+                 const ConstVariableSlot&,
+                 const UString &a_cookie = "") = 0;
+
+    virtual void assign_variable (const VariableSafePtr a_var,
+                                  const UString &a_expression,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void assign_variable
+                    (const VariableSafePtr a_var,
+                     const UString &a_expression,
+                     const ConstVariableSlot &a_slot,
+                     const UString &a_cookie="") = 0;
+
+    virtual void evaluate_variable_expr (const VariableSafePtr a_var,
+                                         const UString &a_cookie = "") = 0;
+    virtual void evaluate_variable_expr
+            (const VariableSafePtr a_var,
+             const ConstVariableSlot &a_slot,
+             const UString &a_cookie = "")= 0;
+
+    virtual void list_changed_variables (VariableSafePtr a_root,
+                                         const UString &a_cookie = "") = 0;
+    virtual void list_changed_variables
+            (VariableSafePtr a_root,
+             const ConstVariableListSlot &a_slot,
+             const UString &a_cookie="") = 0;
+
+    virtual void query_variable_path_expr (const VariableSafePtr a_var,
+                                           const UString &a_cookie = "") = 0;
+    virtual void query_variable_path_expr (const VariableSafePtr a_var,
+                                           const ConstVariableSlot &a_slot,
+                                           const UString &a_cookie = "") = 0;
 
 };//end IDebugger
 
